@@ -7,7 +7,7 @@ from multiprocessing import freeze_support
 from .langserver import LangServer
 from .jsonrpc import JSONRPC2Connection, ReadWriter, path_from_uri
 from .parse_fortran import fortran_file, process_file
-__version__ = '1.11.1'
+__version__ = '1.12.0'
 
 
 def error_exit(error_str):
@@ -82,11 +82,15 @@ def main():
     )
     parser.add_argument(
         '--max_line_length', type=int, default=-1,
-        help="Maximum line length (default: disabled)"
+        help="Maximum line length (default: none)"
     )
     parser.add_argument(
         '--max_comment_line_length', type=int, default=-1,
-        help="Maximum comment line length (default: disabled)"
+        help="Maximum comment line length (default: none)"
+    )
+    parser.add_argument(
+        '--disable_diagnostics', action="store_true",
+        help="Disable diagnostics"
     )
     parser.add_argument(
         '--debug_log', action="store_true",
@@ -188,7 +192,8 @@ def main():
         "sort_keywords": (not args.preserve_keyword_order),
         "enable_code_actions": (args.enable_code_actions or args.debug_actions),
         "max_line_length": args.max_line_length,
-        "max_comment_line_length": args.max_comment_line_length
+        "max_comment_line_length": args.max_comment_line_length,
+        "disable_diagnostics": args.disable_diagnostics,
     }
     if args.hover_language is not None:
         settings["hover_language"] = args.hover_language
