@@ -96,6 +96,7 @@ class LangServer:
         self.hover_language = settings.get("hover_language", "fortran90")
         self.sort_keywords = settings.get("sort_keywords", True)
         self.enable_code_actions = settings.get("enable_code_actions", False)
+        self.disable_diagnostics = settings.get("disable_diagnostics", False)
         self.max_line_length = settings.get("max_line_length", -1)
         self.max_comment_line_length = settings.get("max_comment_line_length", -1)
         # Set object settings
@@ -221,6 +222,7 @@ class LangServer:
                     self.excl_suffixes = config_dict.get("excl_suffixes", [])
                     self.lowercase_intrinsics = config_dict.get("lowercase_intrinsics", self.lowercase_intrinsics)
                     self.debug_log = config_dict.get("debug_log", self.debug_log)
+                    self.disable_diagnostics = config_dict.get("disable_diagnostics", self.disable_diagnostics)
                     self.pp_suffixes = config_dict.get("pp_suffixes", None)
                     self.pp_defs = config_dict.get("pp_defs", {})
                     self.include_dirs = config_dict.get("include_dirs", [])
@@ -423,7 +425,7 @@ class LangServer:
                 return type
 
         def set_type_mask(def_value):
-            return [def_value if i < 8 else True for i in range(15)]
+            return [def_value if i < 8 else True for i in range(16)]
 
         def get_candidates(scope_list, var_prefix, inc_globals=True,
                            public_only=False, abstract_only=False, no_use=False):
@@ -1234,7 +1236,8 @@ class LangServer:
             self.link_version = (self.link_version + 1) % 1000
             for _, file_obj in self.workspace.items():
                 file_obj.ast.resolve_links(self.obj_tree, self.link_version)
-        self.send_diagnostics(uri)
+        if not self.disable_diagnostics:
+            self.send_diagnostics(uri)
 
     def update_workspace_file(self, filepath, read_file=False, allow_empty=False, update_links=False):
         # Update workspace from file contents and path
